package server

import (
	"crypto/rand"
	"encoding/hex"
	"fmt"
	"html/template"
	"log"
	"net/http"
	"strconv"
	"strings"
	"time"

	"streifling.com/jason/sicherheitsunterweisung/packages/data"
	"streifling.com/jason/sicherheitsunterweisung/packages/types"
)

type questionData struct {
	ID int64
	Q  types.Question
	I  int
	J  int
}

func DisplayTable(db *data.DB) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		bs, err := db.GetAllOverviewTableData()
		if err != nil {
			http.Error(w, "DisplayTable: *DB.GetAllOverviewTableData(): "+fmt.Sprint(err), http.StatusInternalServerError)
		}
		template.Must(template.ParseFiles("templates/table.html")).ExecuteTemplate(w, "content", bs)
	}
}

func DisplaySearchResults(db *data.DB) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		bs, err := db.GetOverviewTableDataByName(r.PostFormValue("search"))
		if err != nil {
			http.Error(w, "DisplayResults: db.ReadByName(r.PostFormValue()): "+fmt.Sprint(err), http.StatusInternalServerError)
		}
		template.Must(template.ParseFiles("templates/table.html")).ExecuteTemplate(w, "rows", bs)
	}
}

func DisplayForm(i *int64) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		template.Must(template.ParseFiles("templates/briefing.html")).ExecuteTemplate(w, "content", i)
	}
}

func generateUUID() (string, error) {
	bs := make([]byte, 2)

	if _, err := rand.Read(bs); err != nil {
		return "", fmt.Errorf("GenerateUUID: rand.Read(bs): %v\n", err)
	}

	return hex.EncodeToString(bs), nil
}

func AddParticipant(i *int64, sl *[]string) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		uuid, err := generateUUID()
		if err != nil {
			http.Error(w, "AddParticipant: generateUUID(): "+fmt.Sprint(err), http.StatusInternalServerError)
		}

		*i++
		login := fmt.Sprintf("%d", *i) + "-" + uuid
		(*sl) = append(*sl, login)
		template.Must(template.ParseFiles("templates/briefing.html")).ExecuteTemplate(w, "new", login)
	}
}

// TODO: Hier weiter machen, irgendwie die b.ID herausgeben,
// am besten hier auch die p.IDs rausgeben, damit diese später verknüpft werden können
func SubmitBriefingForm(sb *[]types.Briefing) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		now := time.Now()
		briefing := new(types.Briefing)

		// TODO: Dropdownmenü
		// instructorFirstName := r.PostFormValue("instructor-first")
		// instructorLastName := r.PostFormValue("instructor-last")

		briefing.Date = now.Format("2006-01-02")
		briefing.Time = now.Format("15:04:05")
		briefing.Location = r.PostFormValue("location")
		briefing.DocumentName = r.PostFormValue("document") // TODO: in HTML einfügen
		briefing.AsOf = r.PostFormValue("state")            // TODO: Umbenennen
		// briefing.InstructorID = r.PostFormValue("instructor-id") // TODO: aus Dropdown holen
		(*sb) = append(*sb, *briefing)
	}
}

// TODO: Make it only serve one purpose
func loginIsCorrect(l string, logins *[]string) bool {
	for i, v := range *logins {
		if l == v {
			(*logins) = append((*logins)[:i], (*logins)[i+1:]...)
			return true
		}
	}
	return false
}

func newParticipant(l string) (*types.Participant, error) {
	var err error
	p := new(types.Participant)

	p.ID, err = strconv.Atoi(strings.Split(l, "-")[0])
	if err != nil {
		return nil, fmt.Errorf("newParticipant: strconv.Atoi(idString): %v\n", err)
	}

	return p, nil
}

func DisplayParticipantForm(ls *[]string, cp chan<- *types.Participant) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		l := r.PostFormValue("login")

		if loginIsCorrect(l, ls) {
			p, err := newParticipant(l)
			if err != nil {
				http.Error(w, "GetParticipantData: newParticipant(l): "+fmt.Sprint(err), http.StatusInternalServerError)
			}

			cp <- p
			template.Must(template.ParseFiles("templates/participant.html")).ExecuteTemplate(w, "content", p.ID)
		} else {
			template.Must(template.ParseFiles("templates/login.html")).ExecuteTemplate(w, "content", nil)
		}
	}
}

func readAnswer(r *http.Request, p *types.Participant, i int) error {
	v, err := strconv.Atoi(r.PostFormValue("answer"))
	if err != nil {
		return fmt.Errorf("readAnswer: strconv.Atoi(r.PostFormValue(\"answer\")): %v\n", err)
	}

	p.Questions[i].Chosen = v

	return nil
}

func DisplayQuestion(i int, p *types.Participant) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		if i == 0 {
			p.FirstName = r.PostFormValue("participant-first-" + fmt.Sprintf("%d", p.ID))
			p.LastName = r.PostFormValue("participant-last-" + fmt.Sprintf("%d", p.ID))
			p.Company = r.PostFormValue("participant-company-" + fmt.Sprintf("%d", p.ID))
		} else {
			if err := readAnswer(r, p, i-1); err != nil {
				http.Error(w, "DisplayQuestion: readAnswer(r, p, i): "+fmt.Sprint(err), http.StatusInternalServerError)
			}
		}

		data := new(questionData)
		data.ID = p.ID
		data.Q = p.Questions[i]
		data.I = i
		data.J = i + 1

		template.Must(template.ParseFiles("templates/question.html")).ExecuteTemplate(w, "content", data)
	}
}

func DisplayTestResults(b *types.Briefing, p *types.Participant) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		numQuestions := len(p.Questions)
		wrongAnswers := make([]int, 0)
		fmt.Println(wrongAnswers)

		if err := readAnswer(r, p, numQuestions-1); err != nil {
			http.Error(w, "DisplayTestResults: readAnswer(r, p, i): "+fmt.Sprint(err), http.StatusInternalServerError)
		}

		for i, q := range p.Questions {
			if q.Chosen != q.Correct {
				wrongAnswers = append(wrongAnswers, i)
			}
		}

		if wrongAnswers == nil {
			b.Participants = append(b.Participants, p)
		} else {
			data := new(questionData)
			data.ID = p.ID
			data.Q = p.Questions[0]
			data.I = 0
			data.J = data.I + 1
			template.Must(template.ParseFiles("templates/question.html")).ExecuteTemplate(w, "content", data)
		}

		template.Must(template.ParseFiles("templates/results.html")).ExecuteTemplate(w, "content", nil)
	}
}
